// ==============0ooo===================================================0ooo===========
// =  Copyright (C) 2014-2019 Gowin Semiconductor Technology Co.,Ltd.
// =					 All rights reserved.
// ====================================================================================
// 
//  __	    __	    __
//  \ \	   /  \	   / /   [File name   ] video_top.v
//   \ \  / /\ \  / /	 [Description ] Video demo
//	  \ \/ /  \ \/ /	 [Timestamp   ] Friday May 26 14:00:30 2019
//	   \  /	   \  /	     [version	  ] 1.0.0
//	    \/	    \/
//
// ==============0ooo===================================================0ooo===========
// Code Revision History :
// ----------------------------------------------------------------------------------
// Ver:	|  Author	| Mod. Date	| Changes Made:
// ----------------------------------------------------------------------------------
// V1.0	| Caojie	 | 11/22/19	 | Initial version 
// ----------------------------------------------------------------------------------
// ==============0ooo===================================================0ooo===========

`include "vfb_defines.v"

//`define DRAM_NUM   2'd2

module video_top
(
	input             I_clk           , //50Mhz 
	input             I_key1          ,
    input             I_key2          ,
	output     [3:0]  O_led           ,  
	inout	          IO_adv7513_scl  ,
	inout	          IO_adv7513_sda  ,
	output            O_adv7513_clk    ,
	output            O_adv7513_vs    ,
	output            O_adv7513_hs    ,
	output            O_adv7513_de    ,
	output     [23:0] O_adv7513_data  ,
    output     [13:0] O_ddr_addr      ,
    output     [2:0]  O_ddr_ba        ,
    output            O_ddr_cs_n      ,
    output            O_ddr_ras_n     ,
    output            O_ddr_cas_n     ,
    output            O_ddr_we_n      ,
    output            O_ddr_clk       ,
    output            O_ddr_clk_n     ,
    output            O_ddr_cke       ,
    output            O_ddr_odt       ,
    output            O_ddr_reset_n   ,
    output [2*DRAM_NUM-1:0]     O_ddr_dqm       ,      //DM_WIDTH=2           
    inout  [16*DRAM_NUM-1:0]    IO_ddr_dq       ,      //DQ_WIDTH=16          
    inout  [2*DRAM_NUM-1:0]     IO_ddr_dqs      ,      //DQS_WIDTH=2          
    inout  [2*DRAM_NUM-1:0]     IO_ddr_dqs_n    ,      //DQS_WIDTH=2          
    //
	input             i_rx_clk          ,
	input             i_v_sync          ,
    input             i_d_sync          ,
    input      [15:0] i_pixel_din       
);
localparam DRAM_NUM = 4'd2;

//=========================================================
//SRAM parameters
parameter ADDR_WIDTH          =`DEF_ADDR_WIDTH;        //存储单元是byte，总容量=2^27*16bit = 2Gbit,增加1位rank地址，{rank[0],bank[2:0],row[13:0],cloumn[9:0]}
parameter DATA_WIDTH          =`DEF_SRAM_DATA_WIDTH;   //与生成DDR3IP有关，此ddr3 2Gbit, x16， 时钟比例1:4 ，则固定128bit
parameter WR_VIDEO_WIDTH      =`DEF_WR_VIDEO_WIDTH;  
parameter RD_VIDEO_WIDTH      =`DEF_RD_VIDEO_WIDTH; 

//==================================================
wire        rst_n1;
wire        rst_n2;
wire        ddr_rstn;

//--------------------------
reg  [31:0] run_cnt;
wire        running;

//--------------------------
wire        pixel_clk/* synthesis syn_keep=1 */;
wire        pix_clk_div2;

//--------------------------
wire        tp0_vs_in  ;
wire        tp0_hs_in  ;
wire        tp0_de_in ;
wire [ 7:0] tp0_data_r;
wire [ 7:0] tp0_data_g;
wire [ 7:0] tp0_data_b;

reg         vs_r;
reg  [9:0]  cnt_vs;

//-------------------------
//frame buffer in
wire                      ch0_vfb_clk_in ;
wire                      ch0_vfb_vs_in  ;
wire                      ch0_vfb_de_in  ;
wire [WR_VIDEO_WIDTH-1:0] ch0_vfb_data_in;    

//-----------------------------
//ddr
wire        dma_clk  ;  

//-------------------------------------------------
//memory interface
wire                   memory_clk         ;
wire                   memory_clk_div     ;
wire                   pll_lock           ;
wire                   pll_stop           ;
wire                   cmd_ready          ;
wire[2:0]              cmd                ;
wire                   cmd_en             ;
wire[ADDR_WIDTH-1:0]   addr               ;
wire                   wr_data_rdy        ;
wire                   wr_data_en         ;//
wire                   wr_data_end        ;//
wire[DATA_WIDTH-1:0]   wr_data            ;   
wire[DATA_WIDTH/8-1:0] wr_data_mask       ;   
wire                   rd_data_valid      ;  
wire                   rd_data_end        ;//unused 
wire[DATA_WIDTH-1:0]   rd_data            ;   
wire                   init_calib_complete;

//-------------------
//syn_code
wire                      syn_off0_re;  // ofifo read enable signal
wire                      syn_off0_vs;
wire                      syn_off0_hs;
                          
wire                      off0_syn_de  ;
wire [RD_VIDEO_WIDTH-1:0] off0_syn_data;

//------------------------------------------
//ADV7513 I2c
wire         TX_EN_7513;
wire [2:0]   WADDR_7513;
wire [7:0]   WDATA_7513;
wire         RX_EN_7513;
wire [2:0]   RADDR_7513;
wire [7:0]   RDATA_7513;

//==================================================
key_debounceN #
(
	.DEBCNT1 (32'd50000000 ),//1s
	.DEBCNT2 (32'd100000000) //2s
) 
key_debounceN_inst0
(
	.clk       (I_clk   ), 
	.rst_n     (1'b1    ), 
	.key_in    (I_key1  ),//I_rst_n ), 
	.key_n_out1(rst_n1  ),
	.key_n_out2(rst_n2  )
);

key_debounceN #
(
    .DEBCNT1 (32'd50000000 ),//1s
    .DEBCNT2 (32'd100000000) //2s
)
key_debounceN_inst1
(
    .clk       (I_clk    ), 
    .rst_n     (1'b1    ), 
    .key_in    (I_key2  ), 
    .key_n_out1(        ),
    .key_n_out2(ddr_rstn )
);

//===================================================
//LED灯测试
always @(posedge I_clk or negedge rst_n1)
begin
	if(!rst_n1)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd50_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd25_000_000) ? 1'b1 : 1'b0;


assign  O_led[0] = init_calib_complete;
assign  O_led[1] = running;
assign  O_led[2] = rst_n1;
assign  O_led[3] = rst_n2;
 
//===========================================================================
pix_pll pix_pll_inst
(
    .clkout0(pixel_clk), //output clkout
    .clkout1(pix_clk_div2 ),
    .lock   (         ), //output lock
    .reset  (~rst_n1  ), //input reset
    .clkin  (I_clk    ) //input clkin
);

//===========================================================================
//720p
//testpattern testpattern_inst
//(
//    .I_pxl_clk   (pix_clk_div2          ),//pixel clock
//    .I_rst_n     (rst_n1             ),//low active 
//    .I_mode      ({1'b0,cnt_vs[9:7]} ),//data select
//    .I_sqr_width (16'd60             ),
//    .I_single_r  (8'd0               ),
//    .I_single_g  (8'd255             ),
//    .I_single_b  (8'd0               ),                  //1280x720      //1920x1080(148.5MHz)
//    .I_h_total   (16'd1650           ),//hor total time  // 16'd1650     // 16'd2200
//    .I_h_sync    (16'd40             ),//hor sync time   // 16'd40       // 16'd44  
//    .I_h_bporch  (16'd220            ),//hor back porch  // 16'd220      // 16'd148 
//    .I_h_res     (16'd1280           ),//hor resolution  // 16'd1280     // 16'd1920
//    .I_v_total   (16'd750            ),//ver total time  // 16'd750      // 16'd1125
//    .I_v_sync    (16'd5              ),//ver sync time   // 16'd5        // 16'd5   
//    .I_v_bporch  (16'd20             ),//ver back porch  // 16'd20       // 16'd36  
//    .I_v_res     (16'd720            ),//ver resolution  // 16'd720      // 16'd1080
//    .I_hs_pol    (1'b1               ),//HS polarity , 0:negetive ploarity，1：positive polarity
//    .I_vs_pol    (1'b1               ),//VS polarity , 0:negetive ploarity，1：positive polarity
//    .O_V_cnt     (                   ),
//    .O_H_cnt     (                   ),
//    .O_de        (tp0_de_in          ),   
//    .O_hs        (tp0_hs_in          ),
//    .O_vs        (tp0_vs_in          ),
//    .O_data_r    (tp0_data_r         ),   
//    .O_data_g    (tp0_data_g         ),
//    .O_data_b    (tp0_data_b         )
//);

//1080p
testpattern testpattern_inst
(
    .I_pxl_clk   (pix_clk_div2          ),//pixel clock
    .I_rst_n     (rst_n1             ),//low active 
    .I_mode      ({1'b0,cnt_vs[9:7]} ),//data select
//    .I_mode      ({1'b0,3'b100} ),//data select
    .I_sqr_width (16'd60             ),
    .I_single_r  (8'd0               ),
    .I_single_g  (8'd255             ),
    .I_single_b  (8'd0               ),                  //1280x720      //1920x1080(148.5MHz)
    .I_h_total   (16'd2200           ),//hor total time  // 16'd1650     // 16'd2200
    .I_h_sync    (16'd44             ),//hor sync time   // 16'd40       // 16'd44  
    .I_h_bporch  (16'd148            ),//hor back porch  // 16'd220      // 16'd148 
    .I_h_res     (16'd1920           ),//hor resolution  // 16'd1280     // 16'd1920
    .I_v_total   (16'd1125           ),//ver total time  // 16'd750      // 16'd1125
    .I_v_sync    (16'd5              ),//ver sync time   // 16'd5        // 16'd5   
    .I_v_bporch  (16'd36             ),//ver back porch  // 16'd20       // 16'd36  
    .I_v_res     (16'd1080           ),//ver resolution  // 16'd720      // 16'd1080
    .I_hs_pol    (1'b1               ),//HS polarity , 0:negetive ploarity，1：positive polarity
    .I_vs_pol    (1'b1               ),//VS polarity , 0:negetive ploarity，1：positive polarity
    .O_V_cnt     (                   ),
    .O_H_cnt     (                   ),
    .O_de        (tp0_de_in          ),   
    .O_hs        (tp0_hs_in          ),
    .O_vs        (tp0_vs_in          ),
    .O_data_r    (tp0_data_r         ),   
    .O_data_g    (tp0_data_g         ),
    .O_data_b    (tp0_data_b         )
);


//------------------------------------
always@(posedge pix_clk_div2)//clkout )
begin
	vs_r<=tp0_vs_in;
end

always@(posedge pix_clk_div2 or negedge rst_n1)//clkout )
begin
	if(!rst_n1)
		cnt_vs<=0;
	else if(vs_r && !tp0_vs_in) //vs24 falling edge
		cnt_vs<=cnt_vs+1;
end 

     
//========================================================================
    //pattern data A  16bit
//    assign ch0_vfb_clk_in  = pix_clk_div2  ;
//    assign ch0_vfb_vs_in   = ~tp0_vs_in;
//    assign ch0_vfb_de_in   = tp0_de_in;
//    assign ch0_vfb_data_in = {tp0_data_r[7:3],tp0_data_g[7:2],tp0_data_b[7:3]};//RGB565

    assign ch0_vfb_clk_in  = i_rx_clk  ;
    assign ch0_vfb_vs_in   = i_v_sync;
    assign ch0_vfb_de_in   = i_d_sync;
    assign ch0_vfb_data_in = {i_pixel_din[15:8],i_pixel_din[15:8],i_pixel_din[7:0],i_pixel_din[7:0]};
//------------------------------------------------------------------
//SRAM 控制模块 
Video_Frame_Buffer_Top Video_Frame_Buffer_Top_inst
( 
    .I_rst_n              (init_calib_complete ),//rst_n            ),
    .I_dma_clk            (dma_clk          ),   //sram_clk         ),
`ifdef USE_THREE_FRAME_BUFFER 
    .I_wr_halt            (1'd0             ), //1:halt,  0:no halt
    .I_rd_halt            (1'd0             ), //1:halt,  0:no halt
`endif
    // video data input             
    .I_vin0_clk           (ch0_vfb_clk_in   ),
    .I_vin0_vs_n          (ch0_vfb_vs_in    ),//只接收负极性
    .I_vin0_de            (ch0_vfb_de_in    ),
    .I_vin0_data          (ch0_vfb_data_in  ),
    .O_vin0_fifo_full     (                 ),
    // video data output            
    .I_vout0_clk          (pixel_clk        ),
    .I_vout0_vs_n         (~syn_off0_vs     ),//只接收负极性
    .I_vout0_de           (syn_off0_re      ),
    .O_vout0_den          (off0_syn_de      ),
    .O_vout0_data         (off0_syn_data    ),
    .O_vout0_fifo_empty   (                 ),
    // ddr write request
    .I_cmd_ready          (cmd_ready          ),
    .O_cmd                (cmd                ),//0:write;  1:read
    .O_cmd_en             (cmd_en             ),
    .O_addr               (addr               ),//[ADDR_WIDTH-1:0]
    .I_wr_data_rdy        (wr_data_rdy        ),
    .O_wr_data_en         (wr_data_en         ),//
    .O_wr_data_end        (wr_data_end        ),//
    .O_wr_data            (wr_data            ),//[DATA_WIDTH-1:0]
    .O_wr_data_mask       (wr_data_mask       ),
    .I_rd_data_valid      (rd_data_valid      ),
    .I_rd_data_end        (rd_data_end        ),//unused 
    .I_rd_data            (rd_data            ),//[DATA_WIDTH-1:0]
    .I_init_calib_complete(init_calib_complete)
);  

//----------------------------------------------------- 
//ddr3 ip
Gowin_PLL Gowin_PLL_inst
(
    .lock   (pll_lock), //output lock
    .clkout0(), //output clkout
    .clkout1(memory_clk_div), //output clkout1
    .clkout2(memory_clk), //output clkout2
    .clkin  (I_clk), //input clkin
    .reset  (1'b0), //input reset
    .enclk0 (1'b1), //input enclk0
    .enclk1 (1'b1), //input enclk1
    .enclk2 (pll_stop) //input enclk2
);

DDR3_Memory_Interface_Top u_ddr3 
(
    .clk                (memory_clk_div     ),
    .pll_stop           (pll_stop           ), //output pll_stop
    .memory_clk         (memory_clk         ),
    .pll_lock           (pll_lock           ),
    .rst_n              (ddr_rstn           ), //rst_n1
    .cmd_ready          (cmd_ready          ),
    .cmd                (cmd                ),
    .cmd_en             (cmd_en             ),
    .addr               (addr               ),
    .wr_data_rdy        (wr_data_rdy        ),
    .wr_data            (wr_data            ),
    .wr_data_en         (wr_data_en         ),
    .wr_data_end        (wr_data_end        ),
    .wr_data_mask       (wr_data_mask       ),
    .rd_data            (rd_data            ),
    .rd_data_valid      (rd_data_valid      ),
    .rd_data_end        (rd_data_end        ),
    .sr_req             (1'b0               ),
    .ref_req            (1'b0               ),
    .sr_ack             (                   ),
    .ref_ack            (                   ),
    .init_calib_complete(init_calib_complete),
    .clk_out            (dma_clk            ),
    .burst              (1'b1               ),
    // mem interface
    .ddr_rst            (                   ),
    .O_ddr_addr         (O_ddr_addr         ),
    .O_ddr_ba           (O_ddr_ba           ),
    .O_ddr_cs_n         (O_ddr_cs_n         ),
    .O_ddr_ras_n        (O_ddr_ras_n        ),
    .O_ddr_cas_n        (O_ddr_cas_n        ),
    .O_ddr_we_n         (O_ddr_we_n         ),
    .O_ddr_clk          (O_ddr_clk          ),
    .O_ddr_clk_n        (O_ddr_clk_n        ),
    .O_ddr_cke          (O_ddr_cke          ),
    .O_ddr_odt          (O_ddr_odt          ),
    .O_ddr_reset_n      (O_ddr_reset_n      ),
    .O_ddr_dqm          (O_ddr_dqm          ),
    .IO_ddr_dq          (IO_ddr_dq          ),
    .IO_ddr_dqs         (IO_ddr_dqs         ),
    .IO_ddr_dqs_n       (IO_ddr_dqs_n       )
);

//---------------------------------------------------
//输出同步时序模块
wire out_de;
//720p
//syn_gen syn_gen_inst
//(                                   
//    .I_pxl_clk   (pixel_clk       ),//40MHz      //65MHz      //74.25MHz    //148.5
//    .I_rst_n     (rst_n1           ),//800x600    //1024x768   //1280x720    //1920x1080    
//    .I_h_total   (16'd1650        ),// 16'd1056  // 16'd1344  // 16'd1650   // 16'd2200  
//    .I_h_sync    (16'd40          ),// 16'd128   // 16'd136   // 16'd40     // 16'd44   
//    .I_h_bporch  (16'd220         ),// 16'd88    // 16'd160   // 16'd220    // 16'd148   
//    .I_h_res     (16'd1280        ),// 16'd800   // 16'd1024  // 16'd1280   // 16'd1920  
//    .I_v_total   (16'd750         ),// 16'd628   // 16'd806   // 16'd750    // 16'd1125   
//    .I_v_sync    (16'd5           ),// 16'd4     // 16'd6     // 16'd5      // 16'd5      
//    .I_v_bporch  (16'd20          ),// 16'd23    // 16'd29    // 16'd20     // 16'd36      
//    .I_v_res     (16'd720         ),// 16'd600   // 16'd768   // 16'd720    // 16'd1080   
//    .I_rd_hres   (16'd1280        ),
//    .I_rd_vres   (16'd720         ),
//    .I_hs_pol    (1'b1            ),//HS polarity , 0:负极性，1：正极性
//    .I_vs_pol    (1'b1            ),//VS polarity , 0:负极性，1：正极性
//    .O_rden      (syn_off0_re     ),
//    .O_de        (out_de          ),   
//    .O_hs        (syn_off0_hs     ),
//    .O_vs        (syn_off0_vs     )
//);

//1080
syn_gen syn_gen_inst
(                                   
    .I_pxl_clk   (pixel_clk       ),//40MHz      //65MHz      //74.25MHz    //148.5
    .I_rst_n     (rst_n1           ),//800x600    //1024x768   //1280x720    //1920x1080    
    .I_h_total   (16'd2200        ),// 16'd1056  // 16'd1344  // 16'd1650   // 16'd2200  
    .I_h_sync    (16'd44          ),// 16'd128   // 16'd136   // 16'd40     // 16'd44   
    .I_h_bporch  (16'd148         ),// 16'd88    // 16'd160   // 16'd220    // 16'd148   
    .I_h_res     (16'd1920        ),// 16'd800   // 16'd1024  // 16'd1280   // 16'd1920  
    .I_v_total   (16'd1125        ),// 16'd628   // 16'd806   // 16'd750    // 16'd1125   
    .I_v_sync    (16'd5           ),// 16'd4     // 16'd6     // 16'd5      // 16'd5      
    .I_v_bporch  (16'd36          ),// 16'd23    // 16'd29    // 16'd20     // 16'd36      
    .I_v_res     (16'd1080        ),// 16'd600   // 16'd768   // 16'd720    // 16'd1080   
    .I_rd_hres   (16'd1920        ),
    .I_rd_vres   (16'd1080         ),
    .I_hs_pol    (1'b1            ),//HS polarity , 0:负极性，1：正极性
    .I_vs_pol    (1'b1            ),//VS polarity , 0:负极性，1：正极性
    .O_rden      (syn_off0_re     ),
    .O_de        (out_de          ),   
    .O_hs        (syn_off0_hs     ),
    .O_vs        (syn_off0_vs     )
);

localparam N = 5; //delay N clocks
                          
reg  [N-1:0]  Pout_hs_dn   ;
reg  [N-1:0]  Pout_vs_dn   ;
reg  [N-1:0]  Pout_de_dn   ;

always@(posedge pixel_clk or negedge rst_n1)
begin
    if(!rst_n1)
        begin                          
            Pout_hs_dn  <= {N{1'b1}};
            Pout_vs_dn  <= {N{1'b1}}; 
            Pout_de_dn  <= {N{1'b0}}; 
        end
    else 
        begin                          
            Pout_hs_dn  <= {Pout_hs_dn[N-2:0],syn_off0_hs};
            Pout_vs_dn  <= {Pout_vs_dn[N-2:0],syn_off0_vs}; 
            Pout_de_dn  <= {Pout_de_dn[N-2:0],out_de}; 
        end
end

//---------------------------------------------
//    assign O_adv7513_data    = off0_syn_de ? {off0_syn_data[15:11],3'd0,off0_syn_data[10:5],2'd0,off0_syn_data[4:0],3'd0} : 24'h0000ff;//{r,g,b}
//    assign O_adv7513_vs      = Pout_vs_dn[1];//syn_off0_vs;
//    assign O_adv7513_hs      = Pout_hs_dn[1];//syn_off0_hs;
//    assign O_adv7513_de      = Pout_de_dn[1];//off0_syn_de;
//    assign O_adv7513_clk     = pixel_clk;//pixel_clk_phs;
    
    // assign O_adv7513_data    = off0_syn_de ? off0_syn_data[23:0] : 24'h0000ff;//{r,g,b}
    // assign O_adv7513_vs      = Pout_vs_dn[4];//syn_off0_vs;
    // assign O_adv7513_hs      = Pout_hs_dn[4];//syn_off0_hs;
    // assign O_adv7513_de      = Pout_de_dn[4];//off0_syn_de;
    // assign O_adv7513_clk     = pixel_clk;//pixel_clk_phs;
 
    assign O_adv7513_data    = off0_syn_de ? {off0_syn_data[7:0],off0_syn_data[7:0],off0_syn_data[7:0]} : 24'h0000ff;//{r,g,b}
    assign O_adv7513_vs      = Pout_vs_dn[1];//syn_off0_vs;
    assign O_adv7513_hs      = Pout_hs_dn[1];//syn_off0_hs;
    assign O_adv7513_de      = Pout_de_dn[1];//off0_syn_de;
    assign O_adv7513_clk     = pixel_clk;//pixel_clk_phs;
//=========================================================================
// ADV7513 I2c 初始化
  adv7513_iic_init 	    adv7513_iic_init_inst0
  (
      .I_CLK              ( I_clk                  ),
      .I_RESETN           ( rst_n1                ),//low active
	  .start              ( rst_n2                ),
      .O_TX_EN            ( TX_EN_7513            ),
      .O_WADDR            ( WADDR_7513            ),
      .O_WDATA            ( WDATA_7513            ),
      .O_RX_EN            ( RX_EN_7513            ),
      .O_RADDR            ( RADDR_7513            ),
      .I_RDATA            ( RDATA_7513            ),
      .cstate_flag        (                       ),
      .error_flag         (                       )
  );



  I2C_MASTER_Top        I2C_MASTER_Top_inst0
  (
      .I_CLK              ( I_clk                 ),
      .I_RESETN           ( rst_n2                 ),//low active
      .I_TX_EN            ( TX_EN_7513            ),
      .I_WADDR            ( WADDR_7513            ),
      .I_WDATA            ( WDATA_7513            ),
      .I_RX_EN            ( RX_EN_7513            ),
      .I_RADDR            ( RADDR_7513            ),
      .O_RDATA            ( RDATA_7513            ),
      .O_IIC_INT          (                       ),
      .SCL                ( IO_adv7513_scl           ),
      .SDA                ( IO_adv7513_sda           )
  );

endmodule