//===========================================================================
// Module name: para_config.v
//===========================================================================
//`include "../src/cfg_defines.v"
`include "cfg_defines.v"
`timescale 1ns / 1ps
module para_config 
(  
	input           clk,
	input           clk_com,
	input           rst_n,

	output            SLVS_EC_INCK    ,
	output reg        SLVS_EC_XTRIG1  ,
	output reg        SLVS_EC_XTRIG2  ,
	output reg        SLVS_EC_XCE     ,
	input	            SLVS_EC_SDO     ,
	output reg        SLVS_EC_SDI     ,
	output reg        SLVS_EC_SCK     ,
	output reg        SLVS_EC_OMODE   ,
	output reg        SLVS_EC_SLAMODE ,
	input               SLVS_EC_XHS     ,
	input               SLVS_EC_XVS     ,
	input	            SLVS_EC_TOUT2   ,
	input	            SLVS_EC_TOUT1   ,
	input	            SLVS_EC_TOUT0   ,
	output reg        SLVS_EC_XMASTER ,
	output reg        SLVS_EC_XCLR    ,


	output          led_out
);            

parameter S0      = 16'b0000_0000_0001;
parameter S1      = 16'b0000_0000_0010;
parameter S2      = 16'b0000_0000_0100;
parameter S3      = 16'b0000_0000_1000;
parameter S4      = 16'b0000_0001_0000;
parameter S5      = 16'b0000_0010_0000;
parameter S6      = 16'b0000_0100_0000;
parameter S7      = 16'b0000_1000_0000;
parameter S8      = 16'b0001_0000_0000;
parameter S9      = 16'b0010_0000_0000;

//-----------------------------------------------------------------------------
// Signal definitions
//
reg [15:0]      conf_state; 
reg             SLVS_EC_clk_en;
reg [15:0]      init_delay_cnt; 
reg [31:0]      cfg_delay_cnt; 

//reg              clk_com;
reg [ 3:0]       clk_div_cnt;
reg [ 3:0]      send_state; 
reg [10:0]       ad_i;
wire[23:0]       dout_o;
reg [23:0]       spi_data;
reg [ 7:0]       spi_data_send;
reg [ 4:0]       spi_send_cnt; 
reg [ 1:0]       spi_send_byte_cnt; 

//clk div  74.25/4 = 18.56MHZ  0.05us
//always @(posedge clk or negedge rst_n)    
//begin
//  if (~rst_n) begin                           
//      clk_div_cnt <= 4'd0;  
//      clk_com <= 1'b0;           
//  end
//  else if (clk_div_cnt == 4'd1) begin     
//      clk_com <= ~clk_com;    
//      clk_div_cnt <= 4'd0;  
//  end                  
//  else
//      clk_div_cnt <= clk_div_cnt + 1'b1;            
//end

assign SLVS_EC_INCK = (SLVS_EC_clk_en)? clk:1'b0;
always @(posedge clk_com or negedge rst_n)
begin
	if(~rst_n)
	begin
        SLVS_EC_XCE <= 1'b0;
        SLVS_EC_SCK <= 1'b0;
        SLVS_EC_SDI <= 1'b0;

        SLVS_EC_XCLR <= 1'b0;
        SLVS_EC_XMASTER <= 1'b0;
        SLVS_EC_XTRIG1 <= 1'b0;
        SLVS_EC_XTRIG2 <= 1'b0;

        SLVS_EC_SLAMODE <= 1'b0;
        SLVS_EC_OMODE <= 1'b1; //SLVS-EC: High
        SLVS_EC_SDI <= 1'b0;
        SLVS_EC_clk_en <= 1'b0;
        init_delay_cnt <= 16'd0;
        cfg_delay_cnt <= 32'd0;
        spi_send_cnt <= 5'd0;
        spi_send_byte_cnt <= 2'd0;
        ad_i <= 11'd0;
        send_state <= 4'd0;
        conf_state <= S0;
	end
	else begin
		case(conf_state)
			S0:
			begin   
                SLVS_EC_XCLR <= 1'b0;
                SLVS_EC_XCE  <= 1'b1;
                SLVS_EC_clk_en <= 1'b0;
                if(init_delay_cnt > `INIT_DELAY_1)begin
                    init_delay_cnt <= 16'd0;
                    conf_state <= S1;
                end
                else begin
                    init_delay_cnt <= init_delay_cnt + 1'b1;
                    conf_state <= S0;
                end
			end
			S1:
			begin   
                SLVS_EC_XCLR <= 1'b1;
                SLVS_EC_XCE  <= 1'b1;
                if(init_delay_cnt > `INIT_DELAY_2)begin
                    init_delay_cnt <= 16'd0;
                    conf_state <= S2;
                end
                else begin
                    init_delay_cnt <= init_delay_cnt + 1'b1;
                    conf_state <= S1;
                end
			end
			S2:
			begin   
                SLVS_EC_XCLR <= 1'b1;
                SLVS_EC_clk_en <= 1'b1;
                if(init_delay_cnt == `INIT_DELAY_3)begin
                    init_delay_cnt <= 16'd0;
                    conf_state <= S3;
                end
                else begin
                    SLVS_EC_XCE  <= 1'b1;
                    init_delay_cnt <= init_delay_cnt + 1'b1;
                    conf_state <= S2;
                end
			end



			S3: //reg set
			begin   
                case(send_state)
                    4'd0:
                    begin 
                        if(ad_i == `CFG_NUM)begin
                            ad_i <= 11'd0;
                            send_state <= 4'd1;

                            spi_data <= `CMD_STANDBY_CANCEL;
                        end
                        else begin
                            ad_i <= ad_i + 1'b1;  
                            spi_data <= dout_o;
                            send_state <= 4'd0;
                        end

                        SLVS_EC_XCE  <= 1'b0;
                        conf_state <= S4;
                    end
                    4'd1:
                    begin 
                        if(cfg_delay_cnt > `CFG_DELAY_1)begin // 40ms
                            cfg_delay_cnt <= 32'd0;
                            send_state <= 4'd2;

                            SLVS_EC_XCE  <= 1'b0;
                            spi_data <= `CMD_XMASTER_START;
                            conf_state <= S4;
                        end
                        else begin
                            cfg_delay_cnt <= cfg_delay_cnt + 1'b1;
                            send_state <= 4'd1;
                        end
                    end
                    4'd2:
                    begin 
                        send_state <= 4'd3;

                        SLVS_EC_XCE  <= 1'b0;
//                        spi_data <= `CMD_READ_ID1;
                        spi_data <= `CMD_READ_STANDBY;
                        conf_state <= S4;
                    end
                    4'd3:
                    begin 
                        send_state <= 4'd4;

                        SLVS_EC_XCE  <= 1'b0;
//                        spi_data <= `CMD_READ_ID2;
                        spi_data <= `CMD_READ_XMASTER;
                        conf_state <= S4;
                    end
                    4'd4:
                    begin 
                        send_state <= 4'd0;
                        conf_state <= S8;
                    end
                default: begin
                    conf_state <= S0;
                end
                endcase

			end
			S4: //spi send
			begin  
                spi_data_send <= spi_data[23:16];
                spi_data <= {spi_data[15:0],8'd0};
                if(spi_send_byte_cnt == 2'd3)begin
                    spi_send_byte_cnt <= 2'd0;
                    conf_state <= S7;
                end
                else begin
                    spi_send_byte_cnt <= spi_send_byte_cnt + 1'b1; 
                    conf_state <= S5;
                end
			end
			S5: //spi send
			begin  
                SLVS_EC_SCK  <= 1'b0; 
                SLVS_EC_SDI <= spi_data_send[0];
                spi_data_send <= {1'b0,spi_data_send[7:1]};
                conf_state <= S6;
			end
			S6: //spi send
			begin  
                SLVS_EC_SCK  <= 1'b1; 
                if(spi_send_cnt == 5'd7)begin
                    spi_send_cnt <= 5'd0;
                    conf_state <= S4;
                end
                else begin
                    spi_send_cnt <= spi_send_cnt + 1'b1; 
                    conf_state <= S5;
                end
			end
			S7: //spi send
			begin  
                SLVS_EC_XCE  <= 1'b1;
                conf_state <= S3;
			end

			S8: //cfg end
			begin  
                conf_state <= S8;
			end

			default: begin
				conf_state <= S0;
            end
		endcase
	end

end




//Gowin_pROM Gowin_pROM_u0(
//    .dout(dout_o), //output [23:0] dout
//    .clk(clk), //input clk
//    .oce(1'b1), //input oce
//    .ce(1'b1), //input ce
//    .reset(~rst_n), //input reset
//    .ad(ad_i) //input [10:0] ad
//);

gowin_rom gowin_rom_u0(
    .clk_i(clk_com), //input clk
    .addr_i(ad_i), //input [10:0] ad
    .dout_o(dout_o) //output [23:0] dout
);



//rev test
//================================================
reg [ 7:0]       spi_rev_data;
reg [ 7:0]       spi_rev_data_bf;
reg [ 4:0]       spi_rev_cnt; 

always @(posedge SLVS_EC_SCK or negedge rst_n)
begin
	if(~rst_n) begin
      spi_rev_data <= 8'd0;  
      spi_rev_data_bf <= 8'd0;   
      spi_rev_cnt <= 4'd0;  
    end
    else if (SLVS_EC_XCE) begin                           
      spi_rev_data_bf <= 8'd0;   
      spi_rev_cnt <= 4'd0;    
    end
    else begin 
//        spi_rev_data_bf <= {spi_rev_data_bf[6:0],SLVS_EC_SDI};
        spi_rev_data_bf <= {SLVS_EC_SDI,spi_rev_data_bf[7:1]};
        if (spi_rev_cnt == 4'd7) begin  
            spi_rev_data <= {SLVS_EC_SDI,spi_rev_data_bf[7:1]};
            spi_rev_cnt <= 4'd0;  
        end                  
        else begin
            spi_rev_cnt <= spi_rev_cnt + 1'b1;   
        end
    end
end


  
endmodule