`include "slvs_rx_define.v"

module top (
	input	          sys_clk         ,
	input	          C7513_CEC_CLK   ,
	input	          SYS_CLK_P       ,
	input	          SYS_CLK_N       ,
	input	          reset_n         ,
    //SLVC-EC
	output            SLVS_EC_INCK    ,
	output            SLVS_EC_XTRIG1  ,
	output            SLVS_EC_XTRIG2  ,
	output            SLVS_EC_XCE     ,
	input	          SLVS_EC_SDO     ,
	output            SLVS_EC_SDI     ,
	output            SLVS_EC_SCK     ,
	output            SLVS_EC_OMODE   ,
	output            SLVS_EC_SLAMODE ,
	input             SLVS_EC_XHS     ,
	input             SLVS_EC_XVS     ,
	input	          SLVS_EC_TOUT2   ,
	input	          SLVS_EC_TOUT1   ,
	input	          SLVS_EC_TOUT0   ,
	output            SLVS_EC_XMASTER ,
	output            SLVS_EC_XCLR    ,
	input             I_key1          ,
    input             I_key2          ,
	inout	          IO_adv7513_scl  ,
	inout	          IO_adv7513_sda  ,
	output            O_adv7513_clk   ,
	output            O_adv7513_vs    ,
	output            O_adv7513_hs    ,
	output            O_adv7513_de    ,
	output     [23:0] O_adv7513_data  ,
    output     [13:0] O_ddr_addr      ,
    output     [2:0]  O_ddr_ba        ,
    output            O_ddr_cs_n      ,
    output            O_ddr_ras_n     ,
    output            O_ddr_cas_n     ,
    output            O_ddr_we_n      ,
    output            O_ddr_clk       ,
    output            O_ddr_clk_n     ,
    output            O_ddr_cke       ,
    output            O_ddr_odt       ,
    output            O_ddr_reset_n   ,
    output [2*DRAM_NUM-1:0]     O_ddr_dqm       ,      //DM_WIDTH=2           
    inout  [16*DRAM_NUM-1:0]    IO_ddr_dq       ,      //DQ_WIDTH=16          
    inout  [2*DRAM_NUM-1:0]     IO_ddr_dqs      ,      //DQS_WIDTH=2          
    inout  [2*DRAM_NUM-1:0]     IO_ddr_dqs_n    ,      //DQS_WIDTH=2    
    // 
    output            led_out1, 
    output            led_out2

);
localparam DRAM_NUM = 4'd2;

wire rx_clk/* synthesis syn_keep=1 */;
//================================================================================
wire slvs_ec_clk;
wire sys_clk200;
TLVDS_IBUF IBUF_u0(
    .O(sys_clk200),
    .I(SYS_CLK_P),
    .IB(SYS_CLK_N)
);
Gowin_PLL_SYS Gowin_PLL_SYS(
    .clkout0(slvs_ec_clk),  
    .clkout1(clk_com),
    .clkin(sys_clk200)  
);

para_config para_config_u0(
    .clk             (slvs_ec_clk    ),   
    .clk_com         (clk_com        ),   
    .rst_n           (reset_n        ),  
    .SLVS_EC_INCK    (SLVS_EC_INCK   ),
    .SLVS_EC_XTRIG1  (SLVS_EC_XTRIG1 ),
    .SLVS_EC_XTRIG2  (SLVS_EC_XTRIG2 ),
    .SLVS_EC_XCE     (SLVS_EC_XCE    ),
    .SLVS_EC_SDO     (SLVS_EC_SDO    ),
    .SLVS_EC_SDI     (SLVS_EC_SDI    ),
    .SLVS_EC_SCK     (SLVS_EC_SCK    ),
    .SLVS_EC_OMODE   (SLVS_EC_OMODE  ),
    .SLVS_EC_SLAMODE (SLVS_EC_SLAMODE),
    .SLVS_EC_XHS     (SLVS_EC_XHS    ),
    .SLVS_EC_XVS     (SLVS_EC_XVS    ),
    .SLVS_EC_TOUT2   (SLVS_EC_TOUT2  ),
    .SLVS_EC_TOUT1   (SLVS_EC_TOUT1  ),
    .SLVS_EC_TOUT0   (SLVS_EC_TOUT0  ),
    .SLVS_EC_XMASTER (SLVS_EC_XMASTER),
    .SLVS_EC_XCLR    (SLVS_EC_XCLR   ),
    .led_out         ()  
);

//================================================================================
led_test led_test_u0(
    .sys_clk    (rx_clk  ),   
    .rst_n      (reset_n ),  
    .led_out    (led_out1)  
);

led_test led_test_u1(
    .sys_clk    (sys_clk ),
    .rst_n      (reset_n ),  
    .led_out    (led_out2)  
);

//==============================================================
//======SLVS-EC RX Module
//==============================================================
wire        crc_state;
wire [15:0] pixel0 ;
wire [15:0] pixel1 ;
wire [15:0] pixel2 ;
wire [15:0] pixel3 ;
wire [15:0] pixel4 ;
wire [15:0] pixel5 ;
wire [15:0] pixel6 ;
wire [15:0] pixel7 ;
wire [15:0] pixel8 ;
wire [15:0] pixel9 ;
wire [15:0] pixel10;
wire [15:0] pixel11;
wire [15:0] pixel12;
wire [15:0] pixel13;
wire [15:0] pixel14;
wire [15:0] pixel15;
wire        pixel_v_sync;
wire        pixel_h_sync;
wire        pixel_d_sync;
wire [ 7:0] pixel_oe    ;


wire [15:0] line_number;
wire w_line_valid_rev;
SerDes_Top slvs_ec_rx_u0(
    .SLVS_EC_RX_Top_o_clk              (rx_clk       )   
    ,.SLVS_EC_RX_Top_i_rst_n           (reset_n      )
    //Configure Register
    ,.SLVS_EC_RX_Top_i_sync_symbol     (8'hAA        )
    ,.SLVS_EC_RX_Top_i_deskew_symbol   (8'h60        )
    ,.SLVS_EC_RX_Top_i_standby_symbol  (8'h03        )
    ,.SLVS_EC_RX_Top_i_idle_code_k     (4'h8         )
    ,.SLVS_EC_RX_Top_i_idle_code       (32'h1CA1A2A3 )
    //Set from app
    ,.SLVS_EC_RX_Top_i_crc_option      (`CRC_OPTION  )
    `ifdef FEC_ENABLE
    ,.SLVS_EC_RX_Top_i_ecc_option      (`ECC_OPTION  )
    `endif
    ,.SLVS_EC_RX_Top_i_lane_num        (`LANE_NUM    )
    ,.SLVS_EC_RX_Top_i_line_length     (`LINE_LENGTH )
    ,.SLVS_EC_RX_Top_i_pixel_bit       (`PIXEL_BIT   )
    ,.SLVS_EC_RX_Top_i_config          (1'b1         )
    ,.SLVS_EC_RX_Top_i_standby         (1'b0         )
    ,.SLVS_EC_RX_Top_i_setup           (1'b1         )
    ,.SLVS_EC_RX_Top_i_phy_sync        (1'b1         )       
    ,.SLVS_EC_RX_Top_o_pixel0          (pixel0       )   
    ,.SLVS_EC_RX_Top_o_pixel1          (pixel1       )   
    ,.SLVS_EC_RX_Top_o_pixel2          (pixel2       )   
    ,.SLVS_EC_RX_Top_o_pixel3          (pixel3       )   
    ,.SLVS_EC_RX_Top_o_pixel4          (pixel4       )   
    ,.SLVS_EC_RX_Top_o_pixel5          (pixel5       )   
    ,.SLVS_EC_RX_Top_o_pixel6          (pixel6       )   
    ,.SLVS_EC_RX_Top_o_pixel7          (pixel7       )   
    ,.SLVS_EC_RX_Top_o_pixel8          (pixel8       )   
    ,.SLVS_EC_RX_Top_o_pixel9          (pixel9       )   
    ,.SLVS_EC_RX_Top_o_pixel10         (pixel10      )   
    ,.SLVS_EC_RX_Top_o_pixel11         (pixel11      )   
    ,.SLVS_EC_RX_Top_o_pixel12         (pixel12      )   
    ,.SLVS_EC_RX_Top_o_pixel13         (pixel13      )   
    ,.SLVS_EC_RX_Top_o_pixel14         (pixel14      )   
    ,.SLVS_EC_RX_Top_o_pixel15         (pixel15      )   
    ,.SLVS_EC_RX_Top_o_v_sync          (pixel_v_sync )   
    ,.SLVS_EC_RX_Top_o_h_sync          (pixel_h_sync )   
    ,.SLVS_EC_RX_Top_o_d_sync          (pixel_d_sync )   
    ,.SLVS_EC_RX_Top_o_crc_state       (crc_state    )   

);

wire        w_pix_clk;
wire        w_v_sync;
wire        w_h_sync;
wire        w_d_sync;
wire [15:0] w_pixel_dout;
img_data_gen img_data_gen_u0(
    .i_clk              (rx_clk            )    
    ,.i_rst_n           (reset_n           )
    ,.i_pixel0          (pixel0[7:0]       )       
    ,.i_pixel1          (pixel1[7:0]       )       
    ,.i_pixel2          (pixel2[7:0]       )       
    ,.i_pixel3          (pixel3[7:0]       )       
    ,.i_pixel4          (pixel4[7:0]       )       
    ,.i_pixel5          (pixel5[7:0]       )       
    ,.i_pixel6          (pixel6[7:0]       )       
    ,.i_pixel7          (pixel7[7:0]       )       
    ,.i_pixel8          (pixel8[7:0]       )       
    ,.i_pixel9          (pixel9[7:0]       )       
    ,.i_pixel10         (pixel10[7:0]      )       
    ,.i_pixel11         (pixel11[7:0]      )       
    ,.i_pixel12         (pixel12[7:0]      )       
    ,.i_pixel13         (pixel13[7:0]      )       
    ,.i_pixel14         (pixel14[7:0]      )       
    ,.i_pixel15         (pixel15[7:0]      )       
    ,.i_v_sync          (pixel_v_sync      )       
    ,.i_h_sync          (pixel_h_sync      )       
    ,.i_d_sync          (pixel_d_sync      )       

    ,.o_v_sync          (w_v_sync          )    
    ,.o_h_sync          (w_h_sync          )    
    ,.o_d_sync          (w_d_sync          )        
    ,.o_pixel_dout      (w_pixel_dout      )  

);

video_top video_top_u0(
    .I_clk          (sys_clk),   
    .I_key1          (I_key1        ),
    .I_key2          (I_key2        ),
    .IO_adv7513_scl  (IO_adv7513_scl),
    .IO_adv7513_sda  (IO_adv7513_sda),
    .O_adv7513_clk   (O_adv7513_clk ),
    .O_adv7513_vs    (O_adv7513_vs  ),
    .O_adv7513_hs    (O_adv7513_hs  ),
    .O_adv7513_de    (O_adv7513_de  ),
    .O_adv7513_data  (O_adv7513_data),
    .O_ddr_addr      (O_ddr_addr    ),
    .O_ddr_ba        (O_ddr_ba      ),
    .O_ddr_cs_n      (O_ddr_cs_n    ),
    .O_ddr_ras_n     (O_ddr_ras_n   ),
    .O_ddr_cas_n     (O_ddr_cas_n   ),
    .O_ddr_we_n      (O_ddr_we_n    ),
    .O_ddr_clk       (O_ddr_clk     ),
    .O_ddr_clk_n     (O_ddr_clk_n   ),
    .O_ddr_cke       (O_ddr_cke     ),
    .O_ddr_odt       (O_ddr_odt     ),
    .O_ddr_reset_n   (O_ddr_reset_n ),
    .O_ddr_dqm       (O_ddr_dqm     ),
    .IO_ddr_dq       (IO_ddr_dq     ),
    .IO_ddr_dqs      (IO_ddr_dqs    ),
    .IO_ddr_dqs_n    (IO_ddr_dqs_n  ),
    .i_rx_clk        (rx_clk        ), 
    .i_v_sync        (SLVS_EC_XVS   ),       
    .i_d_sync        (w_d_sync      ),       
    .i_pixel_din     (w_pixel_dout  ) 

);


endmodule